! depth_elems.f90
! List of elements in the depth below the surface nodes.
! Miroslav Broz (miroslav.broz@email.cz), Aug 1st 2007

module depth_elems_module

contains

subroutine depth_elems(nodes, elems, surface_idx, normals_nodes, dh, depth_idx)

use inside_tetrahedron_module

implicit none

double precision, dimension(:,:) :: nodes, normals_nodes
integer, dimension(:,:) :: elems
integer, dimension(:) :: surface_idx, depth_idx
double precision :: dh

integer :: i,j,l,m
logical :: found
double precision, dimension(3) :: x
double precision, dimension(4,3) :: vertices

write(*,'(a)', advance='no') '# Computing depth_idx:   0.00% '

surface_nodes: do j = 1, size(surface_idx)      ! surface nodes index
  i = surface_idx(j)                            ! all nodes index
  x = nodes(i,:) - dh * normals_nodes(j,:)      ! the interpolation point
  l = 0                                         ! all elems index
  all_elems: do
    l = l + 1
    do m = 1, 4
      vertices(m,:) = nodes(elems(l,m),:)
    enddo
    found = inside_tetrahedron(vertices, x)
    if (found.or.(l.eq.size(elems(:,1)))) exit
  enddo all_elems
  if (found) then
    depth_idx(j) = l
  else
    write(*,'(/,a)') '# depth_elems: Error: no enclosing tetrahedron found!'
    write(*,'(a,i8)') '# problematic_node_number = ', i
    write(*,'(a,3(1x,g14.8))') '# x_interpolation = ', x
    stop
  endif

  write(*,'(1x,f6.2,a)', advance='no') 100.d0*j/size(surface_idx), '%'  ! progress bar

enddo surface_nodes

write(*,*)

return

end subroutine depth_elems

end module depth_elems_module


