! gem.f90
! Gauss elimination method.
! Miroslav Broz (miroslav.broz@email.cz), Aug 8th 2007

module gem_module

contains

subroutine gem(a,b,sing)

! BEWARE, BOTH a, b ARE MODIFIED!
! input:
! a(:,:) 	matrix with coefficients
! b(:)		right side
!
! output:
! b(:)		solution
! sing		.true. if matrix is singular

implicit none

double precision, dimension(:,:) :: a
double precision, dimension(:) :: b
logical :: sing

integer :: i,j,k,n
double precision :: cij
double precision, parameter :: eps=1.d-38

sing = .false.
n = size(b)

! preconditioning
do i=1,n-1
  k=i
  do j=i+1,n
    if (abs(a(i,j)).gt.abs(a(i,k))) k=j
  enddo
  if (k.ne.i) then
    cij=b(i)
    b(i)=b(k)
    b(k)=cij
    do j=i,n
      cij=a(j,i)
      a(j,i)=a(j,k)
      a(j,k)=cij
    enddo
  endif

! diagonalisation
  do j=i+1,n
    if (abs(a(i,i)).gt.eps) then
      cij=a(i,j)/a(i,i)
    else
      sing = .true.
      return
    endif
    b(j)=b(j)-cij*b(i)
    do k=i,n
      a(k,j)=a(k,j)-cij*a(k,i)
    enddo
  enddo
enddo

! back-substitution
do i=n,1,-1
  do j=i+1,n
    b(i)=b(i)-a(j,i)*b(j)
  enddo
  if (abs(a(i,i)).gt.eps) then
    b(i)=b(i)/a(i,i)
  else
    sing = .true.
    return
  endif
enddo

return

end subroutine gem

end module gem_module


