! interp_cartesian.f90
! Interpolate u variable on a cartesian grid
!   (with interp_idx precomputed by interp_elems.f90).
! Miroslav Broz (miroslav.broz@email.cz), Aug 2nd 2007

module interp_cartesian_module

contains

subroutine interp_cartesian(u, nodes, elems, interior_idx, interp_idx, dx, dy, dz, u_interp)

use interp_tetrahedron_module

implicit none

double precision, dimension(:) :: u
double precision, dimension(:,:) :: nodes
integer, dimension(:,:) :: elems
integer, dimension(:) :: interior_idx
integer, dimension(:,:) :: interp_idx
double precision :: dx,dy,dz
double precision, dimension(:,:) :: u_interp

integer :: i,j,k,l,m
double precision, dimension(6,3) :: shift
double precision, dimension(4,3) :: vertices
double precision, dimension(4) :: values
double precision, dimension(3) :: x

shift(1,:) = (/+dx, 0.d0, 0.d0/)
shift(2,:) = (/-dx, 0.d0, 0.d0/)
shift(3,:) = (/0.d0, +dy, 0.d0/)
shift(4,:) = (/0.d0, -dy, 0.d0/)
shift(5,:) = (/0.d0, 0.d0, +dz/)
shift(6,:) = (/0.d0, 0.d0, -dz/)

interior_nodes: do j = 1, size(interior_idx)    ! interior nodes index
  i = interior_idx(j)                           ! all nodes index
  shifts: do k = 1, 6                           ! shifts index
    x = nodes(i,:) + shift(k,:)                 ! the interpolation point
    do m = 1, 4
      l = elems(interp_idx(j,k), m)
      vertices(m,:) = nodes(l,:)
      values(m) = u(l)
    enddo
    u_interp(j,k) = interp_tetrahedron(vertices, values, x)
  enddo shifts
enddo interior_nodes 

return

end subroutine interp_cartesian

end module interp_cartesian_module


