! interp_elems.f90
! List of elements (tetrahedra) for the temperature interpolation.
! Miroslav Broz (miroslav.broz@email.cz), Aug 1st 2007

! 2DO: the extrapolation, if necessary

module interp_elems_module

contains

subroutine interp_elems(nodes, elems, interior_idx, dx, dy, dz, interp_idx)

use inside_tetrahedron_module

implicit none

double precision, dimension(:,:) :: nodes
integer, dimension(:,:) :: elems
integer, dimension(:) :: interior_idx
double precision :: dx,dy,dz
integer, dimension(:,:) :: interp_idx

integer :: i,j,k,l,m
logical :: found
double precision, dimension(6,3) :: shift
double precision, dimension(3) :: x
double precision, dimension(4,3) :: vertices

write(*,'(a)', advance='no') '# Computing interp_idx:   0.00% '

shift(1,:) = (/+dx, 0.d0, 0.d0/)
shift(2,:) = (/-dx, 0.d0, 0.d0/)
shift(3,:) = (/0.d0, +dy, 0.d0/)
shift(4,:) = (/0.d0, -dy, 0.d0/)
shift(5,:) = (/0.d0, 0.d0, +dz/)
shift(6,:) = (/0.d0, 0.d0, -dz/)

interior_nodes: do j = 1, size(interior_idx)    ! interior nodes index
  i = interior_idx(j)                           ! all nodes index
  shifts: do k = 1, 6
    x = nodes(i,:) + shift(k,:)                 ! the interpolation point
    l = 0                                       ! all elems index
    all_elems: do
      l = l + 1
      do m = 1, 4
        vertices(m,:) = nodes(elems(l,m),:)
      enddo
      found = inside_tetrahedron(vertices, x)
      if (found.or.(l.eq.size(elems(:,1)))) exit
    enddo all_elems
    if (found) then
      interp_idx(j,k) = l
    else
      write(*,'(/,a)') '# interp_elems: Error: no enclosing tetrahedron found, would have to extrapolate!'
      write(*,'(a,i8)') '# problematic_node_number = ', i
      write(*,'(a,3(1x,g14.8))') '# x_interpolation = ', x
      write(*,'(a,i8)') '# k = ', k
      stop
    endif
  enddo shifts

  write(*,'(1x,f6.2,a)', advance='no') 100.d0*j/size(interior_idx), '%'  ! progress bar

enddo interior_nodes 

write(*,*)

return

end subroutine interp_elems

end module interp_elems_module


