! normals_faces.f90
! Calculate normals to all surface facets.
! Miroslav Broz (miroslav.broz@email.cz), Jul 31st 2007

! 2DO: modify the algorithm for non-convex shapes!
! -> even/odd # of intersections with the remaining facets!

module normals_faces_module

contains

subroutine normals_faces(nodes, faces, normals, areas)

use vector_product_module
use norm_module

implicit none

double precision, dimension(:,:) :: nodes, normals
integer, dimension(:,:) :: faces
double precision, dimension(:) :: areas

integer :: i
double precision :: n
double precision, dimension(3) :: a,b,c

do i = 1, size(faces(:,1))

  a = nodes(faces(i,2),:) - nodes(faces(i,1),:)
  b = nodes(faces(i,1),:) - nodes(faces(i,3),:) ! 1-3 => normal pointing outwards!
  call vector_product(a, b, c)
  n = norm(c)
  if (abs(n).gt.tiny(n)) then
    c = c / n
  else
    write(*,'(a)') '# Warning: zero vector in normals_faces'
  endif

! this does NOT work generally for non-convex shapes (!)
! but it can work WITHOUT this test, if the mesh is structured suitably by tetgen
  if (dot_product(c, nodes(faces(i,1),:)).lt.0.d0) then
    c = -c
    write(*,'(a,i8)') '# Warning: taking negative normal in normal_faces ', i
  endif

  normals(i,:) = c
  areas(i) = n

enddo

return

end subroutine normals_faces

end module normals_faces_module


